unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, //TeeGDIPlus,
  ExtCtrls, TeeProcs, TeEngine, Chart, Series, TeeTools;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure Chart1Zoom(Sender: TObject);
    procedure Chart1UndoZoom(Sender: TObject);
    procedure Chart1Scroll(Sender: TObject);
    procedure Chart1Resize(Sender: TObject);
    procedure Chart1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);

    procedure FormResize(Sender: TObject);


  private
    { Private declarations }
    procedure DragMarksOnDragged(Sender:TDragMarksTool; Index:Integer;
                                    Button:TMouseButton; Shift: TShiftState;
                                    X,Y: Integer);
    procedure PlaceMarks;
    procedure PlaceMarks2;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses StdCtrls;

{$R *.dfm}

var Chart1: TChart;
    ScrollBar : TScrollBar;
    XOffset, YOffset: Array of Integer;
    XOffsetLogical, YOffsetLogical: Array of Integer;
    MouseDownPos: TPoint;

procedure TForm1.FormCreate(Sender: TObject);
var i: Integer;
begin
  Chart1:=TChart.Create(Self);
  Chart1.Parent:=Self;
  Chart1.Align:=alClient;

  Chart1.Legend.Visible:=false;
  Chart1.View3D:=false;

  with Chart1.AddSeries(TBarSeries) do
  begin
    FillSampleValues;
    ColorEachPoint:=true;
    Marks.Arrow.Color:=clRed;
  end;

  SetLength(XOffset, Chart1[0].Count);
  SetLength(YOffset, Chart1[0].Count);

  SetLength(XOffsetLogical, Chart1[0].Count);
  SetLength(YOffsetLogical, Chart1[0].Count);


  for i:=0 to High(XOffset) do
  begin
    XOffset[i]:=0;
    YOffset[i]:=0;
    XOffsetLogical[i]:=0;
    YOffsetLogical[i]:=0;
  end;

  with Chart1.Tools.Add(TDragMarksTool) as TDragMarksTool do
  begin
    Series:=Chart1[0];
    OnDraggedMark:=DragMarksOnDragged;
  end;

  Scrollbar := TScrollBar.Create(Chart1);

  Chart1.OnZoom:=Chart1Zoom;
  Chart1.OnUndoZoom:=Chart1UndoZoom;
  Chart1.OnScroll:=Chart1Scroll;
  Chart1.OnMouseDown:=Chart1MouseDown;

  Chart1.OnResize :=Chart1Resize;  //new
end;


procedure TForm1.FormResize(Sender: TObject);
begin
// occurs after the form is resized
end;

procedure TForm1.DragMarksOnDragged(Sender:TDragMarksTool; Index:Integer;
                                    Button:TMouseButton; Shift: TShiftState;
                                    X,Y: Integer);
begin
 //This code updates XOffset and YOffset with screen coordinates. But for resize we need
 // logical coordinates.

  if Assigned(Sender.Series) then
  begin
    if (Index>-1) and (Index<Length(XOffset)) then
    begin
      XOffset[Index]:=XOffset[Index]+X-MouseDownPos.X;
      YOffset[Index]:=YOffset[Index]+Y-MouseDownPos.Y;

      //XOffsetLogical[Index]:= ??
      //YOffsetLogical[Index]:= ??
    end;
  end;
end;

procedure TForm1.PlaceMarks;
var s, i, tmpWidth, tmpHeight, tmpX, tmpY: Integer;
begin
// only effective if the mouse was used to zoom or scroll, which is
// never in the RM.  We want resize and scrollbar to work, which do not
// use the mouse, thus XOffset and YOffset are wrong for that.
  Chart1.Draw;

  for s:=0 to Chart1.SeriesCount-1 do
    if Chart1[s] is TBarSeries then
      with Chart1[s] as TBarSeries do
      begin
        for i:=0 to Count-1 do
          with Marks.Positions[i] do
          begin
            tmpX:=CalcXPos(i);
            tmpY:=CalcYPos(i);

            tmpWidth:=Canvas.TextWidth(ValueMarkText[i])+12;
            tmpHeight:=Canvas.TextHeight(ValueMarkText[i])+2;
            Custom:=true;
            LeftTop.X:=tmpX + BarWidth div 2 - tmpWidth div 2 + XOffset[i];
            LeftTop.Y:=tmpY - Marks.ArrowLength - tmpHeight + YOffset[i];
            ArrowFrom:=Point(tmpX + BarWidth div 2, tmpY);
            ArrowTo:=Point(LeftTop.X + tmpWidth div 2, LeftTop.Y);
          end;
      end;
end;

procedure TForm1.PlaceMarks2;
var s, i, tmpWidth, tmpHeight, tmpX, tmpY: Integer;
begin
  Chart1.Draw;

  for s:=0 to Chart1.SeriesCount-1 do
    if Chart1[s] is TBarSeries then
      with Chart1[s] as TBarSeries do
      begin
        for i:=0 to Count-1 do
          with Marks.Positions[i] do
          begin
            tmpX:=CalcXPos(i);
            tmpY:=CalcYPos(i);

            tmpWidth:=Canvas.TextWidth(ValueMarkText[i])+12;
            tmpHeight:=Canvas.TextHeight(ValueMarkText[i])+2;
            Custom:=true;
            LeftTop.X:=tmpX + BarWidth div 2 - tmpWidth div 2 + XOffset[i];
            LeftTop.Y:=tmpY - Marks.ArrowLength - tmpHeight + YOffset[i];
            ArrowFrom:=Point(tmpX + BarWidth div 2, tmpY);
            ArrowTo:=Point(LeftTop.X + tmpWidth div 2, LeftTop.Y);
          end;
      end;
end;



procedure TForm1.Chart1Resize(Sender: TObject);
begin
  // added this to example to deal with resize, but it doesn't work
  // because the XScrollPos and YScrollPos are not updated, since
  // there was no mouse click.
  PlaceMarks;
end;

procedure TForm1.Chart1Scroll(Sender: TObject);
begin
//this is for the weird scroll way. You hold the mouse down and drag the
//direction you want to scroll.
  PlaceMarks;
end;

procedure TForm1.Chart1Zoom(Sender: TObject);
begin
//this is for the weird zoom way. You hold the mouse down, drag from top left to bottom down
//and when you release, the rectangle area you've defined is the zoom area.

  PlaceMarks;
end;

procedure TForm1.Chart1UndoZoom(Sender: TObject);
begin
//this is for the weird undo zoom way. You hold the mouse down, drag from bottom down to top left
//and when you release, supposedly the zoom is undone (resets axis scale).

  PlaceMarks;
end;

procedure TForm1.Chart1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
// In the act of zooming or scrolling, this is needed to calculate
// XScroll YScroll so it moves with each mouse point.
// When we resize, or scroll with a scrollbar, this is not called
// so original PlaceMarks doesn't work!
  MouseDownPos.X:=X;
  MouseDownPos.Y:=Y;
end;


end.
